"""
This module contains utility functions for calculating astronomical observations.

The main function in this module is `get_observation`, which calculates the observation period for a specific location and date, and returns an Observation object. The observation period is based on the sunrise and sunset times for the given location and date.

Functions:
    get_observation(configuration: ObservatoryConfig, a_datetime: datetime) -> Observation:
        Calculate the observation period for a specific location and date, and return an Observation object.

"""

from datetime import datetime, timedelta
import logging as log
from typing import Tuple

from suntime import Sun

from ..configuration.configuration import ObservatoryConfig
from ..configuration.core_configuration import LocationConfig
from .data import Observation, ObservationData, Period


def get_observation(
    configuration: ObservatoryConfig, a_datetime: datetime
) -> Observation:
    """
    Calculate the observation period for a specific location and date, and return an Observation object.

    This function calculates the observation period based on the sunrise and sunset times for the given location and date.
    It then creates an Observation object with the calculated start and end times.

    Args:
        configuration (ObservatoryConfig): The configuration object containing the location for which to calculate the observation period.
        a_datetime (datetime): The date for which to calculate the observation period.

    Returns:
        Observation: An Observation object with the calculated start and end times.
    """
    location = configuration.device.location
    log.debug("getting observation for %s @ %s", a_datetime, location.to_json())

    _, sunset_yesterday = __get_sun_data(location, a_datetime - timedelta(days=1))
    sunrise_today, sunset_today = __get_sun_data(location, a_datetime)
    sunrise_tomorrow, _ = __get_sun_data(location, a_datetime + timedelta(days=1))

    if sunrise_today.timestamp() < a_datetime.timestamp():
        log.debug("current day of observation we are after sunrise today")
        log.debug("start is sunset today and sunrise tomorrow")
        observation_start = sunset_today
        observation_end = sunrise_tomorrow
    else:
        log.debug("current date has moved to next day we are before sunrise")
        log.debug("start is sunset yesterday and sunrise today")
        observation_start = sunset_yesterday
        observation_end = sunrise_today

    period = Period(observation_start, observation_end)
    log.debug("observation start %s", period.start)
    log.debug("observation end %s", period.end)
    log.debug("observation title %s", period.date)
    log.debug(
        "are we within an observation period : %s",
        period.within_observation_period(a_datetime),
    )

    data = ObservationData(configuration.nsp.data, configuration.data, period)
    log.debug("data root is %s", data.root_path)
    log.debug("observation root is %s", data.path)
    return Observation(period, data)


def __get_sun_data(
    location: LocationConfig, for_datetime: datetime
) -> Tuple[datetime, datetime]:
    """
    Get the sunrise and sunset times for a specific location and date.

    This function uses the Sun class to calculate the local sunrise and sunset times
    for the given location and date.

    Args:
        location (LocationConfig): The location for which to get the sun data. This should be an instance of LocationConfig.
        for_datetime (datetime): The date for which to get the sun data.

    Returns:
        tuple: A tuple containing the sunrise and sunset times as datetime objects.

    """
    latitude, longitude = location.data()
    sun = Sun(latitude, longitude)
    log.debug("getting sun data for %s @ %s", location.to_json(), for_datetime)
    sunrise = sun.get_local_sunrise_time(for_datetime)
    sunset = sun.get_local_sunset_time(for_datetime)
    return sunrise, sunset



from .configuration.configuration import build_configuration
import logging as log
from datetime import datetime, timezone
from .observation.utils import get_observation
from .observation.filesystem import setup_observation_filesystem
from .housekeeping import perform_housekeeping
from .packaging import perform_packaging
from .capture.imaging import perform_observation

from time import sleep


def run(arguments):

    while True:
        configuration = build_configuration(arguments.configuration)

        current_datetime = datetime.now()
        log.debug("starting the root loop")
        log.debug("current datetime is %s", current_datetime)
        observation = get_observation(configuration, current_datetime)

        if observation.period.within_observation_period(current_datetime):
            log.info(
                "observation %s till %s",
                observation.period.date,
                observation.period.end,
            )
            observation = setup_observation_filesystem(observation)
            perform_observation(observation, configuration)
            log.info(
                "waiting configured time before performing additional tasks for %s minutes",
                configuration.nsp.observation_cooldown,
            )
            sleep(configuration.nsp.observation_cooldown * 60)
        else:
            log.info("not within observation period")
            perform_housekeeping(configuration)
            perform_packaging(configuration)
            seconds_till_observation = (
                observation.period.calculate_wait_till_observation(
                    datetime.now(timezone.utc)
                )
            )
            log.debug("%s seconds till observation start", seconds_till_observation)
            log.info("waiting for observation period at %s", observation.period.start)
            sleep(seconds_till_observation)
        log.debug("ending the root loop")

        if arguments.test_mode:  # in test no infinite loop
            break



from dataclasses import dataclass
from typing import Optional
from dataclass_wizard import JSONWizard
from .core_configuration import LoggingConfig, DataConfig


@dataclass
class ModuleLoggingConfig(LoggingConfig):
    file: Optional[str] = None

    def merge(self, logging_config: LoggingConfig):
        if not self.path:
            self.path = logging_config.path
        if not self.level:
            self.level = logging_config.level
        if not self.format:
            self.format = logging_config.format
        if not self.rotation:
            self.rotation = logging_config.rotation


@dataclass
class HouseKeeping(JSONWizard):
    delete_after: int
    unit: Optional[str] = "DAYS"

    def __post_init__(self):
        if self.delete_after < 1.0:
            raise ValueError("Delete after must be greater or equal than 1.0")

    def get_age(self) -> int:
        if self.unit == "DAYS":
            return self.delete_after * 86400


@dataclass
class ModuleDataConfig(DataConfig):
    house_keeping: Optional[HouseKeeping] = None



import paho.mqtt.client as mqtt
from ..configuration.core_configuration import MQTTConfig
import json
import logging as log


def publish_message(config: MQTTConfig, topic: str, message: dict) -> None:
    """
    Publishes a message to an MQTT broker.

    Args:
        config (MQTTConfig): The MQTT configuration object.
        topic (str): The topic to publish the message to.
        message (str): The message to be published.

    Returns:
        None
    """
    if config.enabled:
        client = mqtt.Client()
        client.connect(config.host, 1883, 60)
        client.publish(topic, str(json.dumps(message)))
        client.disconnect()
        log.debug("published message to %s", topic)



import json
import logging as log
from ..utilities.conversions import microsecond_to_seconds
from ..observation.data import Observation
from ..configuration.nsp_configuration import Capture


def create_json_file(
    observation: Observation, capture: Capture, file_name: str, image_format: str
) -> dict:
    json_data = {
        "observation": {
            "date": observation.period.date,
            "start": observation.period.start.isoformat(),
            "end": observation.period.end.isoformat(),
        },
        "data": {
            "path": observation.data_config.path,
            "root_path": observation.data_config.root_path,
            "observation_image_path": observation.data_config.observation_image_path,
            "observation_data_path": observation.data_config.observation_data_path,
        },
        "exposure": {
            "shutter": microsecond_to_seconds(capture.shutter.current),
            "gain": capture.gain.current,
            "white_balance": {
                "red": capture.white_balance.red,
                "blue": capture.white_balance.blue,
            },
        },
        "image": {
            "path": f"{observation.data_config.observation_image_path}{file_name}{image_format}",
            "format": image_format,
            "filename": file_name,
        },
    }

    output_file = f"{observation.data_config.observation_data_path}{file_name}.json"
    log.debug("Creating JSON file: %s", output_file)
    try:
        with open(output_file, "w") as json_file:
            json.dump(json_data, json_file)
        log.debug("JSON file created successfully: %s", output_file)
        return json_data
    except Exception as e:
        log.error("Failed to create JSON file: %s", e)



from dataclasses import dataclass
from typing import Optional
from dataclass_wizard import JSONWizard


@dataclass
class LocationConfig(JSONWizard):
    latitude: float
    longitude: float

    def __post_init__(self):
        if self.latitude > 90 or self.latitude < -90:
            raise ValueError("Invalid Location:Latitude")
        if self.longitude > 180 or self.longitude < -180:
            raise ValueError("Invalid Location:Longitude")

    def data(self):
        return self.latitude, self.longitude


@dataclass
class MQTTConfig(JSONWizard):
    enabled: Optional[bool] = False
    host: Optional[str] = "mqtt://localhost"


@dataclass
class DeviceConfig(JSONWizard):
    name: str
    location: LocationConfig
    mqtt: MQTTConfig


@dataclass
class LoggingFormatConfig(JSONWizard):
    date: str
    output: str


@dataclass
class LoggingRotationConfig(JSONWizard):
    size: int
    backup: int


@dataclass
class LoggingConfig(JSONWizard):
    path: Optional[str] = None
    level: Optional[str] = None
    format: Optional[LoggingFormatConfig] = None
    rotation: Optional[LoggingRotationConfig] = None


@dataclass
class DataConfig(JSONWizard):
    path: str



import logging as log
from typing import List
from .configuration.configuration import ObservatoryConfig
import os, shutil
import pathlib
from .utilities.mqtt_client import publish_message


def perform_packaging(config: ObservatoryConfig) -> None:
    log.debug("starting the packaging process")
    data_location = f"{config.data.path}/{config.nsp.data.path}"

    if not is_packaging_required(data_location):
        log.debug("no packaging required")
        log.info("packaging processes completed")
        return

    for item in data_directory_contents(config):
        log.info("packaging item %s", item)
        folder_name = pathlib.PurePath(item).name
        package_format = "zip"
        __zip_folder(item, data_location, folder_name, package_format)
        publish_message(
            config=config.device.mqtt,
            topic="nsp/archive-completed",
            message={
                "name": folder_name,
                "format": package_format,
                "folder": data_location,
                "path": f"{data_location}/{folder_name}.{package_format}",
                "notification": f"{config.device.name} : Archive completed for {folder_name}",
            },
        )
        __delete_folder(item)
        log.info("packaging of %s completed", item)

    log.info("packaging processes completed")


def data_directory_contents(config: ObservatoryConfig) -> List[str]:
    log.debug("scanning data directory for folders to compress")
    data_location = f"{config.data.path}/{config.nsp.data.path}"
    log.debug("data directory set to %s", data_location)
    directories = []
    for path in sorted(os.listdir(data_location)):
        log.debug("checking item %s for compression", path)
        if os.path.isdir(os.path.join(data_location, path)):
            log.debug("%s is a directory directory will be compressed", path)
            directories.append(os.path.join(data_location, path))

    log.debug("found %s directories that need compression", len(directories))
    return directories


def is_packaging_required(data_location: str) -> bool:

    if not os.path.exists(data_location):
        log.warning("data root path %s does not exist", data_location)
        return False

    log.debug("checking if packaging is required")
    log.debug("checking root folder %s", data_location)

    processing_required = False
    for path in sorted(os.listdir(data_location)):
        log.debug("checking item %s for processing", path)
        if os.path.isdir(os.path.join(data_location, path)):
            log.debug("%s is a directory processing will be required", path)
            processing_required = True

    return processing_required


def __zip_folder(source, root, base, package_format) -> None:
    log.debug("about to compress directory %s into %s.%s", base, base, package_format)
    shutil.make_archive(source, package_format, root, base)
    log.info("completed compression of %s into %s.%s", base, base, package_format)


def __delete_folder(path: str) -> None:
    log.debug("deleting %s", path)
    shutil.rmtree(path)
    log.debug("directory %s has been deleted", path)



import logging as log
from PIL import Image
from ..configuration.nsp_configuration import Capture
from ..utilities.conversions import microsecond_to_seconds


def calculate_next_exposure_value(image_path, capture: Capture):

    average_brightness = calculate_average_brightness(image_path)

    log.debug("Average brightness: %s", average_brightness)
    current_shutter_speed = capture.shutter.current
    current_gain = capture.gain.current
    brightness_threshold = capture.exposure.target
    tolerance = capture.exposure.tolerance
    lower_threshold = brightness_threshold - tolerance
    upper_threshold = brightness_threshold + tolerance

    new_gain = current_gain
    new_shutter_speed = current_shutter_speed

    brightness_difference = abs(
        max(
            average_brightness - brightness_threshold,
            brightness_threshold - average_brightness,
        )
    )
    log.debug("Brightness difference: %s", brightness_difference)
    # Calculate adjustments
    if average_brightness < lower_threshold:
        # Image is too dark, increase shutter speed and gain
        log.debug("Image is too dark, increasing shutter speed and gain")
        temp_shutter = min(
            current_shutter_speed + (brightness_difference + current_shutter_speed),
            capture.shutter.slowest,
        )
        if temp_shutter < capture.shutter.slowest:
            new_shutter_speed = temp_shutter
        else:
            new_shutter_speed = capture.shutter.slowest
            new_gain = min(
                current_gain + (brightness_difference * current_gain),
                capture.gain.highest,
            )

    elif average_brightness > upper_threshold:
        log.debug("Image too bright, decreasing shutter speed and increasing gain")
        log.debug("current_gain: %s", current_gain)
        log.debug("capture.gain.lowest: %s", capture.gain.lowest)
        if current_gain == capture.gain.lowest:
            log.debug("Decreasing Shutter Speed")
            new_shutter_speed = max(
                current_shutter_speed - (current_shutter_speed * brightness_difference),
                capture.shutter.fastest,
            )
        else:
            log.debug("Decreasing Gain")
            new_gain = max(
                current_gain - (brightness_difference * current_gain),
                capture.gain.lowest,
            )
    else:
        # Image is within the target range
        log.debug("Image is within the target range, no adjustments needed")
        new_shutter_speed = current_shutter_speed
        new_gain = current_gain

    # Update the capture configuration
    capture.shutter.current = new_shutter_speed
    capture.gain.current = new_gain
    log.debug("New shutter speed: %s", microsecond_to_seconds(capture.shutter.current))
    log.debug("New gain: %s", capture.gain.current)
    log.debug("Next exposure value calculated")


def calculate_average_brightness(image_path):
    log.debug("Calculating next exposure value")
    image = Image.open(image_path, formats=["JPEG"])
    crop = image.convert("L")
    histogram = crop.histogram()
    pixels = sum(histogram)
    brightness = scale = len(histogram)

    for index in range(0, scale):
        ratio = histogram[index] / pixels
        brightness += ratio * (-scale + index)

    average_brightness = 1 if brightness == 255 else round(brightness / scale, 2)
    return average_brightness



from ..configuration.configuration import ObservatoryConfig
from ..configuration.core_configuration import MQTTConfig
from ..observation.data import Observation
from .reporting import create_json_file
from .exposure import calculate_next_exposure_value
from datetime import datetime
import logging as log
from time import sleep, time
import subprocess
from ..configuration.nsp_configuration import Capture
from ..utilities.mqtt_client import publish_message


def perform_observation(
    observation: Observation, configuration: ObservatoryConfig
) -> None:
    log.info("starting observation capture period")
    log.debug("creating initial exposure values from configuration")
    capture_configuration: Capture = configuration.nsp.capture
    mqtt_config = configuration.device.mqtt
    publish_message(
        config=mqtt_config,
        topic="nsp/observation-started",
        message=__get_observation_message(configuration, observation),
    )
    while observation.period.within_observation_period(datetime.now()):
        log.debug("within observation starting processes to capture single image")
        capture_configuration = __capture_image(
            observation, capture_configuration, mqtt_config
        )
        delay = configuration.nsp.capture.exposure.delay
        log.debug("sleeping for %s seconds", delay)
        sleep(delay)
    log.info("completed observation capture period")
    publish_message(
        config=mqtt_config,
        topic="nsp/observation-ended",
        message=__get_observation_message(configuration, observation, finished=True),
    )


def __get_observation_message(
    config: ObservatoryConfig, observation: Observation, finished=False
) -> dict:
    json_data = {
        "observation": {
            "date": observation.period.date,
            "start": observation.period.start.isoformat(),
            "end": observation.period.end.isoformat(),
        },
        "data": {
            "path": observation.data_config.path,
            "root_path": observation.data_config.root_path,
            "observation_image_path": observation.data_config.observation_image_path,
            "observation_data_path": observation.data_config.observation_data_path,
        },
        "notification": f"{config.device.name} : Observation {'completed' if finished else 'started'} for {observation.period.date}",
    }
    return json_data


def __capture_image(
    observation: Observation, capture: Capture, mqtt_config: MQTTConfig
) -> Capture:
    log.debug("starting image capture")
    log.debug("capturing image for observation %s", observation.period.date)
    image_name = f"{round(time())}"
    image_format = ".jpg"
    filename = (
        f"{observation.data_config.observation_image_path}{image_name}{image_format}"
    )
    # Construct the command
    exposure_settings = (
        f"--shutter {capture.shutter.current} --gain {capture.gain.current} "
    )
    white_balance = (
        f"--awbgains {capture.white_balance.red},{capture.white_balance.blue} "
    )
    switches = "-n --immediate --denoise cdn_hq "
    command = "libcamera-still "
    
    if capture.size.width != 0 and capture.size.height != 0:
        switches += f"--width {capture.size.width} --height {capture.size.height} "

    call = f"{command} -o {filename} {exposure_settings} {white_balance} {switches}"

    try:
        log.debug("image capture timeout: %s", capture.timeout)
        subprocess.run(
            call,
            shell=True,
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT,
            timeout=capture.timeout,
        )
        log.info("image capture completed")
        json = create_json_file(observation, capture, image_name, image_format)
        publish_message(config=mqtt_config, topic="nsp/image-captured", message=json)
        calculate_next_exposure_value(filename, capture)
    except Exception as e:
        log.error(e)

    return capture



"""
This module defines data classes used in the observation module.

The main class in this module is `Period`, which represents an observation period with a start and end time. It also provides methods to check if a given datetime is within the observation period and to calculate the wait time until the observation period starts.

Classes:
    Period: Represents an observation period with a start and end time.

Functions:
    within_observation_period(for_datetime: datetime) -> bool:
        Check if a given datetime is within the observation period.

    calculate_wait_till_observation(for_datetime: datetime) -> int:
        Calculate the wait time in minutes until the observation period starts.
"""

from dataclasses import dataclass
from datetime import datetime

from dataclasses_json import dataclass_json

from ..configuration.module_configuration import (
    HouseKeeping,
    ModuleDataConfig,
    DataConfig,
)


@dataclass_json
@dataclass(init=False)
class Period:
    """
    Represents an observation period with a start and end time.

    This class is used to represent an observation period for a specific location and date. It provides methods to check if a given datetime is within the observation period and to calculate the wait time until the observation period starts.

    Attributes:
        start (datetime): The start time of the observation period.
        end (datetime): The end time of the observation period.
    """

    start: datetime
    end: datetime
    date: str

    def __init__(self, start: datetime, end: datetime):
        self.start = start
        self.end = end
        self.date = self.start.strftime("%Y-%m-%d")

    def within_observation_period(self, for_datetime: datetime) -> bool:
        """
        Check if a given datetime is within the observation period.

        This method checks if the given datetime is within the start and end times of the observation period.

        Args:
            for_datetime (datetime): The datetime to check.

        Returns:
            bool: True if the datetime is within the observation period, False otherwise.
        """
        timestamp = for_datetime.timestamp()
        return self.start.timestamp() < timestamp < self.end.timestamp()

    def calculate_wait_till_observation(self, for_datetime: datetime) -> int:
        """
        Calculate the wait time in minutes until the observation period starts.

        This method calculates the time difference between the given datetime and the start time of the observation period. If the given datetime is after the start time, the method returns 0.

        Args:
            for_datetime (datetime): The datetime for which to calculate the wait time.

        Returns:
            int: The wait time in minutes until the observation period starts.
        """
        wait_time = max((self.start - for_datetime).total_seconds(), 0)
        return int(wait_time)


@dataclass_json
@dataclass(init=False)
class ObservationData:
    """
    Represents the data collected during an observation period. Along with the paths of the data collected. with Optional housekeeping settings.
    """

    data_path: str
    root_path: str
    path: str
    observation_image_path: str
    observation_data_path: str
    house_keeping: HouseKeeping

    def __init__(
        self, module: ModuleDataConfig, global_data_config: DataConfig, period: Period
    ):
        self.data_path = f"{global_data_config.path}/"
        self.root_path = f"{self.data_path}{module.path}/"
        self.house_keeping = module.house_keeping
        self.path = f"{self.root_path}{period.date}/"
        self.observation_image_path = f"{self.path}images/"
        self.observation_data_path = f"{self.path}data/"


@dataclass_json
@dataclass
class Observation:
    """
    A class used to represent an observation.

    This class encapsulates the data and period of an observation. It uses an instance of the `Period` class to represent the observation period and an instance of the `ObservationData` class to represent the data collected during that period.

    Attributes:
        period (Period): The observation period.
        data_config (ObservationData): The data collected during the observation period.
    """

    period: Period
    data_config: ObservationData



import argparse

from .app.main import run


def night_sky_pi():
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--configuration", type=str, required=True)
    arguments = parser.parse_args(None)
    run(arguments)




"""
This module contains functions for performing housekeeping tasks on the data generated by the observatory.

The main function in this module is `perform_housekeeping`, which checks if housekeeping is enabled and if so, deletes files older than a certain age from the data location.

Functions:
    perform_housekeeping(config: ObservatoryConfig) -> None:
        Perform housekeeping tasks if enabled in the configuration. This includes deleting files older than a certain age from the data location.

    is_housekeeping_enabled(config: ObservatoryConfig) -> bool:
        Check if housekeeping is enabled in the configuration. Returns True if enabled, False otherwise.
"""

import logging as log
import os
import time

import magic

from .configuration.configuration import ObservatoryConfig
from .utilities.mqtt_client import publish_message
from typing import List


def perform_housekeeping(config: ObservatoryConfig) -> None:
    """
    Perform housekeeping tasks if enabled in the configuration.

    This function checks if housekeeping is enabled and if so, deletes files older than a certain age from the data location.
    The age of the files and the data location are specified in the configuration.

    Args:
        config (ObservatoryConfig): The configuration object containing the housekeeping settings.

    Returns:
        None
    """
    log.debug("entering house keeping")
    if not is_housekeeping_enabled(config):
        log.debug("house keeping is disabled")
        return
    log.debug("house keeping is enabled")

    data_location = f"{config.data.path}/{config.nsp.data.path}"
    age = config.nsp.data.house_keeping.get_age()
    for item in get_files_for_deletion(data_location, age):
        log.debug("file %s will be deleted", item)
        os.remove(item)
        publish_message(
            config=config.device.mqtt,
            topic="nsp/file-deleted",
            message={
                "file": item,
                "notification": f"{config.device.name} : File deleted by housekeeping",
            },
        )
        log.info("deleted file %s", item)

    log.info("house keeping completed")


def is_housekeeping_enabled(config: ObservatoryConfig) -> bool:
    """
    Check if housekeeping is enabled in the configuration.

    This function checks if the data root path exists and if housekeeping is enabled in the configuration.

    Args:
        config (ObservatoryConfig): The configuration object containing the housekeeping settings.

    Returns:
        bool: True if housekeeping is enabled and the data root path exists, False otherwise.
    """
    log.debug("checking if house keeping is enabled")
    data_location = f"{config.data.path}/{config.nsp.data.path}"
    if not os.path.exists(data_location):
        log.warn("data root path %s does not exist", data_location)
        return False

    return config.nsp.data.house_keeping


def get_files_for_deletion(date_location, older_than) -> List[str]:
    """
    Get a list of files that are older than a certain age.

    This function scans the given directory and returns a list of files that are older than the specified age.
    Only files with a MIME type of "application/zip" are considered.

    Args:
        date_location (str): The directory to scan for files.
        older_than (int): The age in seconds. Files older than this age will be included in the returned list.

    Returns:
        List[str]: A list of file paths that are older than the specified age.
    """
    log.debug("getting files for deletion")

    files = []

    with os.scandir(date_location) as listOfEntries:
        for entry in listOfEntries:
            if entry.is_dir():
                log.debug("skipping check on %s it's a directory", entry.name)
                continue
            if magic.from_file(entry.path, mime=True) == "application/zip":
                age = time.time() - entry.stat().st_mtime
                log.debug("file %s has age %d ", entry.name, age)
                if age > older_than:
                    log.debug("file %s will be deleted", entry.name)
                    files.append(entry.path)
                    continue
                log.debug("file %s is not old enough for deletion", entry.name)

    log.debug("returning %d files for deletion", len(files))

    return files



from dataclasses import dataclass, field
from typing import Optional
from dataclass_wizard import JSONWizard
from .module_configuration import ModuleDataConfig, ModuleLoggingConfig


@dataclass
class Shutter(JSONWizard):
    initial: Optional[int] = 250000
    slowest: Optional[int] = 15000000
    fastest: Optional[int] = 100
    current: Optional[int] = initial


@dataclass
class Gain(JSONWizard):
    initial: Optional[float] = 1.0
    lowest: Optional[float] = 1.0
    highest: Optional[float] = 8.0
    current: Optional[float] = initial


@dataclass
class WhiteBalance(JSONWizard):
    red: Optional[float] = 2.8
    blue: Optional[float] = 1.7


@dataclass
class Exposure(JSONWizard):
    target: Optional[float] = 100
    delay: Optional[int] = 5
    tolerance: Optional[float] = 0.01


@dataclass
class CaptureFormat(JSONWizard):
    file: Optional[str] = "jpg"


@dataclass
class ImageSize(JSONWizard):
    width: Optional[int] = 0
    height: Optional[int] = 0
    
@dataclass
class Capture(JSONWizard):
    shutter: Shutter = field(default_factory=Shutter)
    white_balance: WhiteBalance = field(default_factory=WhiteBalance)
    exposure: Exposure = field(default_factory=Exposure)
    format: CaptureFormat = field(default_factory=CaptureFormat)
    gain: Gain = field(default_factory=Gain)
    timeout: Optional[int] = 100
    size: ImageSize = field(default_factory=ImageSize)


@dataclass
class NSP(JSONWizard):
    logging: ModuleLoggingConfig
    data: ModuleDataConfig
    capture: Capture = field(default_factory=Capture)
    observation_cooldown: Optional[int] = 10



from dataclasses import dataclass
from dataclass_wizard import JSONWizard
import json
from .core_configuration import DeviceConfig, LoggingConfig, DataConfig
from .nsp_configuration import NSP
from os.path import isfile


from logging.handlers import RotatingFileHandler

import logging
import os


@dataclass
class ObservatoryConfig(JSONWizard):
    device: DeviceConfig
    nsp: NSP
    logging: LoggingConfig
    data: DataConfig


def __configure_logging(configuration: ObservatoryConfig) -> None:
    configuration.nsp.logging.merge(configuration.logging)
    log_conf = configuration.nsp.logging

    if not os.path.exists(log_conf.path):
        os.makedirs(log_conf.path)

    file = f"{log_conf.path}/{log_conf.file}"
    handler = RotatingFileHandler(
        file, maxBytes=log_conf.rotation.size, backupCount=log_conf.rotation.backup
    )

    logging.basicConfig(
        level=log_conf.level,
        format=log_conf.format.output,
        datefmt=log_conf.format.date,
        handlers=[handler],
    )

    logging.info("configuration created")


def __configure_data(configuration: ObservatoryConfig) -> None:
    if not os.path.exists(configuration.data.path):
        logging.info("creating data directory %s", configuration.data.path)
        os.makedirs(configuration.data.path)


def build_configuration(config_path: str) -> ObservatoryConfig:
    if not isfile(config_path):
        raise IOError("Configuration file not found.")
    with open(config_path) as configuration_file:
        file_contents = configuration_file.read()
    configuration = ObservatoryConfig.from_dict(json.loads(file_contents))
    __configure_logging(configuration)
    __configure_data(configuration)
    return configuration



"""
filesystem.py
-------------

This module contains utility functions for setting up and managing the filesystem for observations.

Functions:
    setup_observation_filesystem(observation: Observation) -> Observation:
        Checks if the directory for the given observation exists, and if not, creates it.
"""

import logging as log
import os

from .data import Observation


def setup_observation_filesystem(observation: Observation) -> Observation:
    """
    Checks if the directory for the given observation exists, and if not, creates it.

    Parameters:
        observation (Observation): The observation for which to check and possibly create a directory.

    Returns:
        Observation: The same observation that was passed in. This allows for function chaining.
    """
    log.debug("checking directories for observation %s", observation.period.date)
    if not os.path.exists(observation.data_config.path):
        log.info(
            "creating observation %s directory at %s",
            observation.period.date,
            observation.data_config.path,
        )
        os.makedirs(observation.data_config.path)
        os.makedirs(observation.data_config.observation_image_path)
        os.makedirs(observation.data_config.observation_data_path)

    return observation
